#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaPlotAlignPD31 {Left, Right, DoNotPlot}
public enum anaPivotStylesPD31 {Floor, Wide, Fibonacci, Jackson, Camarilla, Woodies}
public enum anaSessionTypePD31 {ETH, RTH, Custom}
public enum anaSessionCountPD31 {First, Second, Third}

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Pivot Points.
	/// </summary>
	[Description("Daily Session Pivots.")]
	public class anaPivotsDailyV31: Indicator
	{
		#region Variables
		private	SolidBrush[]		brushes					= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),
															new SolidBrush(Color.Black), new SolidBrush(Color.Black)};
		private DateTime 			firstBarSessionBegin	= Cbi.Globals.MinDate;
		private DateTime 			firstBarSessionEnd 		= Cbi.Globals.MinDate;
		private DateTime			plotStart				= Cbi.Globals.MinDate;
		private DateTime			cacheSessionBeginTmp   	= Cbi.Globals.MinDate;
		private DateTime			cacheSessionEndTmp		= Cbi.Globals.MinDate;
		private DateTime			cacheSessionDate		= Cbi.Globals.MinDate;
		private DateTime			currentDate				= Cbi.Globals.MinDate;
		private DateTime			priorDate				= Cbi.Globals.MinDate;
		private	double				currentHigh				= double.MinValue;
		private	double				currentLow				= double.MaxValue;
		private	double				currentClose			= 0;
		private	double				volSum					= 0;
		private	double				volPriceSum				= 0;
		private	double				currentVolSum			= 0;
		private	double				currentVolPriceSum		= 0;
		private double				displaySize				= 0;
		private Bars				dailyBars				= null;
		private SolidBrush			textBrush				= new SolidBrush(Color.Red);
		private Font				textFont				= new Font("Arial", 12);
		private string				errorData1				= "Daily Pivots can only be displayed on intraday charts.";
		private string				errorData2				= "No daily data found. Please reload daily data or use CalcFromIntradayData.";
		private string				errorData3				= "Loading of daily data is incomplete. Please reload daily data or use CalcFromIntradayData.";
		private	string				errorData4				= "Insufficient historical data to calculate pivots. Please increase chart look back period.";
		private	string				errorData5				= "The prior close is not within the range of the selected RTH session.";
        private float				errorTextWidth			= 0;
        private float				errorTextHeight			= 0;
		private bool				existsHistDailyData		= false;
		private bool				isDailyDataLoaded		= false;
		private bool				plotPivots				= true;
		private bool				periodOpen				= false;
		private bool				skipPublicHoliday		= false; 
		private bool				isPublicHoliday			= false; 
		private bool				noDailyData				= false;
		private bool				noCurrentDailyBar		= false;
		private bool				isCurrency				= false;
		private bool				isGlobex				= false;
		private ArrayList			newSessionBarIdxArr1	= new ArrayList();
		private anaPlotAlignPD31	plotLabels				= anaPlotAlignPD31.Right;
		private anaPivotStylesPD31 	pivotFormula			= anaPivotStylesPD31.Floor;
		private anaSessionTypePD31	pivotSession			= anaSessionTypePD31.Custom;
		private anaSessionCountPD31 activeSession			= anaSessionCountPD31.Second;
		private Data.PivotRange		pivotRangeType1			= PivotRange.Daily;
		private HLCCalculationMode	priorDayHLC				= HLCCalculationMode.DailyBars;
		private double				range					= 0;
		private	double				pp						= 0;
		private	double				r1						= 0;
		private	double				r2						= 0;
		private	double				r3						= 0;
		private	double				r4						= 0;
		private	double				s1						= 0;
		private	double				s2						= 0;
		private	double				s3						= 0;
		private	double				s4						= 0;
		private	double				rmid					= 0;
		private	double				r12mid					= 0;
		private	double				r23mid					= 0;
		private	double				r34mid					= 0;
		private	double				smid					= 0;
		private	double				s12mid					= 0;
		private	double				s23mid					= 0;
		private	double				s34mid					= 0;
		private double				midLine					= 0;
		private double				pivotRangeLine			= 0;
		private double				previousVWAP			= 0;
		private double				previousHigh			= 0;
		private double				previousLow				= 0;
		private double				previousClose			= 0;
		private DateTime			sessionDateTmp			= Cbi.Globals.MinDate;
		private DateTime			sessionDateDaily		= Cbi.Globals.MinDate;
		private	StringFormat		stringFormatCenter		= new StringFormat();
		private	StringFormat		stringFormatFar			= new StringFormat();
		private	StringFormat		stringFormatNear		= new StringFormat();
		private double				userDefinedClose		= 0;
		private double				userDefinedHigh			= 0;
		private double				userDefinedLow			= 0;
		private int					countDown				= 2;
		private int					numberOfSessions		= 1;
		private int					sessionCount			= 0;
		private int					width					= 15;
		private int 				labelPosition			= 10;
		private int					startBar				= 0;
		private bool				rthPivots				= false;
		private bool				showMidpivots			= false;
		private DateTime			publicHoliday0			= new DateTime (2009,01,19);
		private DateTime			publicHoliday1			= new DateTime (2009,02,16);
		private DateTime			publicHoliday2			= new DateTime (2009,05,25);
		private DateTime			publicHoliday3			= new DateTime (2009,07,03);
		private DateTime			publicHoliday4			= new DateTime (2009,09,07);
		private DateTime			publicHoliday5			= new DateTime (2009,11,26);
		private DateTime			publicHoliday6			= new DateTime (2010,01,18);
		private DateTime			publicHoliday7			= new DateTime (2010,02,15);
		private DateTime			publicHoliday8			= new DateTime (2010,05,31);
		private DateTime			publicHoliday9			= new DateTime (2010,07,05);
		private DateTime			publicHoliday10			= new DateTime (2010,09,06);
		private DateTime			publicHoliday11			= new DateTime (2010,11,25);
		private DateTime			publicHoliday12			= new DateTime (2011,01,17);
		private DateTime			publicHoliday13			= new DateTime (2011,02,21);
		private DateTime			publicHoliday14			= new DateTime (2011,05,30);
		private DateTime			publicHoliday15			= new DateTime (2011,07,04);
		private DateTime			publicHoliday16			= new DateTime (2011,09,05);
		private DateTime			publicHoliday17			= new DateTime (2011,11,24);
		private DateTime[]			publicHoliday			= new DateTime [18];
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"PP "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R1 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S1 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R2 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S2 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R3 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S3 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R4 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S4 "));
			
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"RMid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"SMid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R12Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S12Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R23Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S23Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"R34Mid "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"S34Mid "));
			
			Add(new Plot(new Pen(Color.Lime,2), PlotStyle.Line,"Y-High "));
			Add(new Plot(new Pen(Color.OrangeRed,2), PlotStyle.Line,"Y-Low "));
			Add(new Plot(new Pen(Color.Yellow,2), PlotStyle.Line,"Y-Close "));
			Add(new Plot(new Pen(Color.MediumSpringGreen,1), PlotStyle.Line,"Y-Mid "));
			Add(new Plot(new Pen(Color.MediumSpringGreen,1), PlotStyle.Line,"Y-PRL "));

			Add(new Plot(new Pen(Color.HotPink,1), PlotStyle.Line,"Y-VWAP"));

			Plots[0].Pen.DashStyle = DashStyle.Solid;
			Plots[1].Pen.DashStyle = DashStyle.Solid;
			Plots[2].Pen.DashStyle = DashStyle.Solid;
			Plots[3].Pen.DashStyle = DashStyle.Solid;
			Plots[4].Pen.DashStyle = DashStyle.Solid;
			Plots[5].Pen.DashStyle = DashStyle.Solid;
			Plots[6].Pen.DashStyle = DashStyle.Solid;
			Plots[7].Pen.DashStyle = DashStyle.Solid;
			Plots[8].Pen.DashStyle = DashStyle.Solid;
			Plots[9].Pen.DashStyle = DashStyle.Solid;
			Plots[10].Pen.DashStyle = DashStyle.Solid;
			Plots[11].Pen.DashStyle = DashStyle.Solid;
			Plots[12].Pen.DashStyle = DashStyle.Solid;
			Plots[13].Pen.DashStyle = DashStyle.Solid;
			Plots[14].Pen.DashStyle = DashStyle.Solid;
			Plots[15].Pen.DashStyle = DashStyle.Solid;
			Plots[16].Pen.DashStyle = DashStyle.Solid;
			Plots[17].Pen.DashStyle = DashStyle.Dot;
			Plots[18].Pen.DashStyle = DashStyle.Dot;
			Plots[19].Pen.DashStyle = DashStyle.Dot;
			Plots[20].Pen.DashStyle = DashStyle.Solid;
			Plots[21].Pen.DashStyle = DashStyle.Solid;
			Plots[22].Pen.DashStyle = DashStyle.Solid;
			
			AutoScale						= false;
			Overlay							= true;
			PriceTypeSupported				= false;
			ZOrder							= 1;
			stringFormatNear.Alignment 		= StringAlignment.Near;
			stringFormatCenter.Alignment 	= StringAlignment.Center;
			stringFormatFar.Alignment		= StringAlignment.Far;
		}

		/// <summary>
		/// </summary>
		protected override void OnStartUp()
		{
			if (Instrument.MasterInstrument.InstrumentType == InstrumentType.Future &&
				(Instrument.MasterInstrument.Name == "EMD" ||Instrument.MasterInstrument.Name == "ES" || Instrument.MasterInstrument.Name == "NQ" 
				||Instrument.MasterInstrument.Name == "YM"||Instrument.MasterInstrument.Name == "GE" ||Instrument.MasterInstrument.Name == "SR" 
				||Instrument.MasterInstrument.Name == "UB"||Instrument.MasterInstrument.Name == "ZB" ||Instrument.MasterInstrument.Name == "ZF"
				||Instrument.MasterInstrument.Name == "ZN"||Instrument.MasterInstrument.Name == "ZQ" ||Instrument.MasterInstrument.Name == "ZT"
				||Instrument.MasterInstrument.Name == "6A"||Instrument.MasterInstrument.Name == "6B" ||Instrument.MasterInstrument.Name == "6C"
				||Instrument.MasterInstrument.Name == "6E"||Instrument.MasterInstrument.Name == "6J" ||Instrument.MasterInstrument.Name == "6M"
				||Instrument.MasterInstrument.Name == "6N"||Instrument.MasterInstrument.Name == "6S" ||Instrument.MasterInstrument.Name == "E7"
				||Instrument.MasterInstrument.Name == "J7"||Instrument.MasterInstrument.Name == "M6A" ||Instrument.MasterInstrument.Name == "M6B" 
				||Instrument.MasterInstrument.Name == "M6C"||Instrument.MasterInstrument.Name == "M6E" ||Instrument.MasterInstrument.Name == "M6J"
				||Instrument.MasterInstrument.Name == "M6S"||Instrument.MasterInstrument.Name == "CL" ||Instrument.MasterInstrument.Name == "EH"
				||Instrument.MasterInstrument.Name == "GC"||Instrument.MasterInstrument.Name == "HG" ||Instrument.MasterInstrument.Name == "HO"
				||Instrument.MasterInstrument.Name == "NG"||Instrument.MasterInstrument.Name == "QG" ||Instrument.MasterInstrument.Name == "QM"
				||Instrument.MasterInstrument.Name == "RB"||Instrument.MasterInstrument.Name == "SI" ||Instrument.MasterInstrument.Name == "YG" 
				||Instrument.MasterInstrument.Name == "YI"||Instrument.MasterInstrument.Name == "GF" ||Instrument.MasterInstrument.Name == "GPB"
				||Instrument.MasterInstrument.Name == "HE"||Instrument.MasterInstrument.Name == "LE" ||Instrument.MasterInstrument.Name == "YC"
				||Instrument.MasterInstrument.Name == "YK"||Instrument.MasterInstrument.Name == "YW" ||Instrument.MasterInstrument.Name == "ZC"
				||Instrument.MasterInstrument.Name == "ZE"||Instrument.MasterInstrument.Name == "ZL" ||Instrument.MasterInstrument.Name == "ZM"
				||Instrument.MasterInstrument.Name == "ZO"||Instrument.MasterInstrument.Name == "ZR" ||Instrument.MasterInstrument.Name == "ZS"
				||Instrument.MasterInstrument.Name == "ZW"))				
				isGlobex = true;
			if (isGlobex)
			{
				publicHoliday[0] = publicHoliday0;
				publicHoliday[1] = publicHoliday1;
				publicHoliday[2] = publicHoliday2;
				publicHoliday[3] = publicHoliday3;
				publicHoliday[4] = publicHoliday4;
				publicHoliday[5] = publicHoliday5;
				publicHoliday[6] = publicHoliday6;
				publicHoliday[7] = publicHoliday7;
				publicHoliday[8] = publicHoliday8;
				publicHoliday[9] = publicHoliday9;
				publicHoliday[10] = publicHoliday10;
				publicHoliday[11] = publicHoliday11;
				publicHoliday[12] = publicHoliday12;
				publicHoliday[13] = publicHoliday13;
				publicHoliday[14] = publicHoliday14;
				publicHoliday[15] = publicHoliday15;
				publicHoliday[16] = publicHoliday16;
				publicHoliday[17] = publicHoliday17;
			}			
			if (Instrument.MasterInstrument.InstrumentType == Cbi.InstrumentType.Currency || Instrument.MasterInstrument.Name == "6A"
				|| Instrument.MasterInstrument.Name == "6B" || Instrument.MasterInstrument.Name == "6C" ||Instrument.MasterInstrument.Name == "6E"
				|| Instrument.MasterInstrument.Name == "6J" || Instrument.MasterInstrument.Name == "6M" || Instrument.MasterInstrument.Name == "6S"
				|| Instrument.MasterInstrument.Name == "6N" || Instrument.MasterInstrument.Name == "E7" || Instrument.MasterInstrument.Name == "J7"
				|| Instrument.MasterInstrument.Name == "M6A" || Instrument.MasterInstrument.Name == "M6B" || Instrument.MasterInstrument.Name == "M6C"
				|| Instrument.MasterInstrument.Name == "M6E" || Instrument.MasterInstrument.Name == "M6J" || Instrument.MasterInstrument.Name == "M6S")
				isCurrency = true;
			
			if (pivotSession == anaSessionTypePD31.ETH || (pivotSession == anaSessionTypePD31.Custom && isCurrency))
				rthPivots = false;
			else
				rthPivots = true;
			if (PivotFormula == anaPivotStylesPD31.Jackson)
			{
				Plots[0].Name = "BP";
				Plots[1].Name = "D";
				Plots[2].Name = "B";
				Plots[3].Name = "RB1";
				Plots[4].Name = "SB1";
				Plots[5].Name = "D1";
				Plots[6].Name = "B1";
				Plots[7].Name = "RB2";
				Plots[8].Name = "SB2";
			}
			else
			{
				Plots[0].Name = "PP";
				Plots[1].Name = "R1";
				Plots[2].Name = "S1";
				Plots[3].Name = "R2";
				Plots[4].Name = "S2";
				Plots[5].Name = "R3";
				Plots[6].Name = "S3";
				Plots[7].Name = "R4";
				Plots[8].Name = "S4";
			}
			if (AutoScale)
				AutoScale = false;
			if (Instrument.MasterInstrument.InstrumentType == Cbi.InstrumentType.Currency)
			{	
				if (TickSize == 0.00001 || TickSize == 0.001)
					displaySize = 5* TickSize;
			}
			else
				displaySize = TickSize;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (Bars == null)
				return; 
			if (!Bars.BarsType.IsIntraday && Bars.Period.Id != PeriodType.Day)
				return;
			if (Bars.Period.Id == PeriodType.Day && Bars.Period.Value > 1)
				return;
			if (Bars.Period.Id == PeriodType.Day && pivotRangeType1 == PivotRange.Daily)
			{
				textBrush.Color = ChartControl.AxisColor;
				DrawTextFixed("errortag1", errorData1, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
				return;
			}			

			if (CurrentBar == 0)
			{	
				Bars.Session.GetNextBeginEnd(Time[0], out firstBarSessionBegin, out firstBarSessionEnd);
				plotStart = firstBarSessionEnd;
				plotStart = plotStart.AddDays(1);
				if (plotStart.DayOfWeek == DayOfWeek.Saturday)
					plotStart = plotStart.AddDays(2);
			}
			
			if (!isDailyDataLoaded)
			{
				if (priorDayHLC == HLCCalculationMode.DailyBars && Bars.BarsType.IsIntraday) 
				{
					Enabled = false;
					System.Threading.ThreadPool.QueueUserWorkItem(new System.Threading.WaitCallback(GetBarsNow));
					return;
				}
				existsHistDailyData = false;
				isDailyDataLoaded	= true;
			}
			if (priorDayHLC == HLCCalculationMode.DailyBars && !existsHistDailyData)
			{
				textBrush.Color = ChartControl.AxisColor;
				DrawTextFixed("errortag2", errorData2, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
				return;
			}	

			IBar dailyBar;
			if (existsHistDailyData) 
			{
				noCurrentDailyBar = false;
				dailyBar = dailyBars.Get(dailyBars.GetBar(currentDate));
				if (dailyBar.Time.Date != currentDate)
				{
					noCurrentDailyBar = true;
					dailyBar = dailyBars.Get(dailyBars.GetBar(priorDate));
					if (dailyBar.Time.Date != priorDate)
					{
						noDailyData = true;
					}	
				}
			} 
			else 
				dailyBar = null;

			double	high		= (existsHistDailyData && !rthPivots) ? dailyBar.High : High[0];
			double	low			= (existsHistDailyData && !rthPivots) ? dailyBar.Low : Low[0];
			double	close		= existsHistDailyData ? dailyBar.Close : Close[0];

			DateTime lastBarTimeStamp1 = GetLastBarSessionDate(Time[0], Bars, pivotRangeType1);
			if (lastBarTimeStamp1 != currentDate || priorDayHLC == HLCCalculationMode.UserDefinedValues)
			{
				countDown = countDown -1;
				sessionCount = 1;
				startBar = CurrentBar;
				periodOpen = false;
				if (countDown <1 && existsHistDailyData)
				{	
					currentClose = close;
					if (!rthPivots)
					{
						currentHigh = Math.Max(currentHigh, high);
						currentLow = Math.Min(currentLow, low);
					}
				}
				priorDate = currentDate;
				currentDate = lastBarTimeStamp1;
				skipPublicHoliday = false;
				for (int i =0; i<18; i++)
				{
					if (publicHoliday[i].Date == priorDate)
						skipPublicHoliday = true;
				}
				if ((countDown<0 && !skipPublicHoliday) || priorDayHLC == HLCCalculationMode.UserDefinedValues)
				{
					range = (currentHigh - currentLow);						
					if (PivotFormula == anaPivotStylesPD31.Fibonacci)
					{
						pp				= (currentHigh + currentLow + currentClose)/ 3;
						s1				= pp - 0.382*range;
						r1				= pp + 0.382*range;
						s2				= pp - 0.618*range;
						r2				= pp + 0.618*range;
						s3				= pp - range;
						r3				= pp + range;
						s4				= pp - 1.382*range;
						r4				= pp + 1.382*range;
					}
					else if (PivotFormula == anaPivotStylesPD31.Jackson)
					{
						pp				= (currentHigh + currentLow + currentClose)/ 3;
						s1				= pp - 0.500*range;
						r1				= pp + 0.500*range;
						s2				= pp - 0.618*range;
						r2				= pp + 0.618*range;
						s3				= pp - range;
						r3				= pp + range;
						s4				= pp - 1.382*range;
						r4				= pp + 1.382*range;
					}
					else if (PivotFormula == anaPivotStylesPD31.Camarilla)
					{
						pp				= currentClose;
						s1				= currentClose - 1.1/12*range;
						r1				= currentClose + 1.1/12*range;
						s2				= currentClose - 1.1/6*range;
						r2				= currentClose + 1.1/6*range;
						s3				= currentClose - 1.1/4*range;
						r3				= currentClose + 1.1/4*range;
						s4				= currentClose - 1.1/2*range;
						r4				= currentClose + 1.1/2*range;
					}
					else
					{
						if (PivotFormula == anaPivotStylesPD31.Floor || PivotFormula == anaPivotStylesPD31.Wide)
							pp				= (currentHigh + currentLow + currentClose)/ 3;
						else if (PivotFormula == anaPivotStylesPD31.Woodies)
							pp				= (currentHigh + currentLow + 2*currentClose)/ 4;
						
						s1				= 2 * pp - currentHigh;
						r1				= 2 * pp - currentLow;
						s2				= pp - range;
						r2				= pp + range;
						if (PivotFormula == anaPivotStylesPD31.Floor || PivotFormula == anaPivotStylesPD31.Woodies)
						{
							s3				= s1 - range;
							r3				= r1 + range;
							s4				= s3 - (currentHigh - pp);
							r4				= r3 + (pp - currentLow);
						}
						else if (PivotFormula == anaPivotStylesPD31.Wide)
						{
							s3				= s2 - range;
							r3				= r2 + range;
							s4				= s3 - range;
							r4				= r3 + range;
						}
						smid			= 0.5*(pp+s1);
						rmid			= 0.5*(pp+r1);
						s12mid			= 0.5*(s1+s2);
						r12mid			= 0.5*(r1+r2);
						s23mid			= 0.5*(s2+s3);
						r23mid			= 0.5*(r2+r3);
						s34mid			= 0.5*(s3+s4);
						r34mid			= 0.5*(r3+r4);
					}
					previousHigh		= currentHigh;
					previousLow			= currentLow;
					previousClose		= currentClose;
					midLine				= (currentHigh + currentLow) / 2;
					pivotRangeLine		= 2*pp - midLine;
					if (volSum > double.Epsilon)
						previousVWAP	= currentVolPriceSum/currentVolSum;
					else
						previousVWAP	= 0;
					if (noDailyData)
						plotPivots = false;
					else 
						plotPivots = true;
					noDailyData = false;
				}	
				if (!existsHistDailyData || (existsHistDailyData && rthPivots)) 
				{
					if (numberOfSessions == 1 || !rthPivots || (rthPivots && activeSession == anaSessionCountPD31.First))
					{
						if (!existsHistDailyData)	
							currentClose		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
						if (!skipPublicHoliday || skipPublicHoliday && rthPivots)
						{
							currentHigh			= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : high;
							currentLow			= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : low;
							volSum				= 0;
							volPriceSum			= 0;
						}
						else
						{
							currentHigh			= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
							currentLow			= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
							if (FirstTickOfBar)
							{
								volSum			= currentVolSum;
								volPriceSum		= currentVolPriceSum;
							}
						}
						currentVolSum		= volSum + Volume[0];
						currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
						periodOpen 		= true;
					}
					else 
					{	
						volSum				= 0;
						volPriceSum			= 0;
						currentVolPriceSum 	= 0;
						currentVolSum		= 0;
						periodOpen			= false;
					}
				}
				else  
				{	
					if (!skipPublicHoliday || noCurrentDailyBar)
					{
						currentHigh 	= double.MinValue;
						currentLow 		= double.MaxValue;
					}
					if (!skipPublicHoliday)
					{
							volSum				= 0;
							volPriceSum			= 0;
					}
					else
					{
							if (FirstTickOfBar)
							{
								volSum			= currentVolSum;
								volPriceSum		= currentVolPriceSum;
							}
					}
					currentVolSum		= volSum + Volume[0];
					currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
					periodOpen 			= true;
				}
			}
			else if (Bars.FirstBarOfSession)
			{
				if (FirstTickOfBar)
				{	
					sessionCount = sessionCount +1;
					numberOfSessions = Math.Min(3, Math.Max(sessionCount, numberOfSessions));
				}
				if (existsHistDailyData && !rthPivots) 
				{
					if (FirstTickOfBar)
					{
						volSum			= currentVolSum;
						volPriceSum		= currentVolPriceSum;
					}
					currentVolSum		= volSum + Volume[0];
					currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
				}	
				else if (!existsHistDailyData && !rthPivots)
				{
						currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
						currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
						currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
						if (FirstTickOfBar)
						{
							volSum			= currentVolSum;
							volPriceSum		= currentVolPriceSum;
						}
						currentVolSum		= volSum + Volume[0];
						currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
						periodOpen			= true;
				}
				else if (rthPivots && ((sessionCount == 2 && activeSession == anaSessionCountPD31.Second)
					|| (sessionCount == 3 && activeSession == anaSessionCountPD31.Third)))
				{
					if (!existsHistDailyData)	
						currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
					currentHigh			= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : high;
					currentLow			= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : low;
					volSum				= 0;
					volPriceSum			= 0;
					currentVolSum		= Volume[0];
					currentVolPriceSum	= Typical[0]*Volume[0];
					periodOpen			= true;
				}
				else 
					periodOpen = false;
			}
			else if (periodOpen)
			{	
				if (!existsHistDailyData || (existsHistDailyData && rthPivots)) 
				{
					if (!existsHistDailyData)
						currentClose	= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedClose : close;
					currentHigh		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedHigh : Math.Max(currentHigh, high);
					currentLow		= (priorDayHLC == HLCCalculationMode.UserDefinedValues) ? userDefinedLow : Math.Min(currentLow, low);
				}
				if (FirstTickOfBar)
				{
					volSum			= currentVolSum;
					volPriceSum		= currentVolPriceSum;
				}
				currentVolSum		= volSum + Volume[0];
				currentVolPriceSum	= volPriceSum + Typical[0]*Volume[0];
			}
			
			if (plotPivots && currentDate != Cbi.Globals.MinDate && !(rthPivots && activeSession == anaSessionCountPD31.Third && numberOfSessions == 2))
			{
				pp = Math.Round(pp/displaySize)* displaySize;
				r1 = Math.Round(r1/displaySize)* displaySize;
				s1 = Math.Round(s1/displaySize)* displaySize;
				r2 = Math.Round(r2/displaySize)* displaySize;
				s2 = Math.Round(s2/displaySize)* displaySize;
				r3 = Math.Round(r3/displaySize)* displaySize;
				s3 = Math.Round(s3/displaySize)* displaySize;
				r4 = Math.Round(r4/displaySize)* displaySize;
				s4 = Math.Round(s4/displaySize)* displaySize;
				if (PivotFormula != anaPivotStylesPD31.Camarilla)
					PP.Set(pp);
				R1.Set(r1);
				S1.Set(s1);
				R2.Set(r2);
				S2.Set(s2);
				R3.Set(r3);
				S3.Set(s3);
				R4.Set(r4);
				S4.Set(s4);

				if (showMidpivots && PivotFormula != anaPivotStylesPD31.Camarilla && 
					PivotFormula != anaPivotStylesPD31.Fibonacci && PivotFormula != anaPivotStylesPD31.Jackson)
				{
					rmid = Math.Round(rmid/displaySize)* displaySize;
					smid = Math.Round(smid/displaySize)* displaySize;
					r12mid = Math.Round(r12mid/displaySize)* displaySize;
					s12mid = Math.Round(s12mid/displaySize)* displaySize;
					r23mid = Math.Round(r23mid/displaySize)* displaySize;
					s23mid = Math.Round(s23mid/displaySize)* displaySize;
					r34mid = Math.Round(r34mid/displaySize)* displaySize;
					s34mid = Math.Round(s34mid/displaySize)* displaySize;
					RMid.Set(rmid);
					SMid.Set(smid);
					R12Mid.Set(r12mid);
					S12Mid.Set(s12mid);
					R23Mid.Set(r23mid);
					S23Mid.Set(s23mid);
					R34Mid.Set(r34mid);
					S34Mid.Set(s34mid);
				}	
				midLine = Math.Round(midLine/displaySize)* displaySize;
				pivotRangeLine = Math.Round(pivotRangeLine/displaySize)* displaySize;
				PreviousHigh.Set(previousHigh);
				PreviousLow.Set(previousLow);
				PreviousClose.Set(previousClose);
				MidLine.Set(midLine);
				PivotRangeLine.Set(pivotRangeLine);
				if (Instrument.MasterInstrument.InstrumentType != Cbi.InstrumentType.Index && 
				 	Instrument.MasterInstrument.InstrumentType != Cbi.InstrumentType.Currency && previousVWAP > double.Epsilon
					&& priorDayHLC != HLCCalculationMode.UserDefinedValues)
				{	
					previousVWAP = Math.Round(previousVWAP/displaySize)* displaySize;
					PVWAP.Set(previousVWAP);
				}
				else
					PVWAP.Reset();
			}
			else
			{	
				PP.Set(0);
				R1.Set(0);
				S1.Set(0);
				R2.Set(0);
				S2.Set(0);
				R3.Set(0);
				S3.Set(0);
				R4.Set(0);
				S4.Set(0);
				RMid.Set(0);
				SMid.Set(0);
				R12Mid.Set(0);
				S12Mid.Set(0);
				R23Mid.Set(0);
				S23Mid.Set(0);
				R34Mid.Set(0);
				S34Mid.Set(0);
				PreviousHigh.Set(0);
				PreviousLow.Set(0);
				PreviousClose.Set(0);
				MidLine.Set(0);
				PivotRangeLine.Set(0);
				PVWAP.Set(0);
			}
			if (existsHistDailyData && (rthPivots || priorDayHLC == HLCCalculationMode.UserDefinedValues) 
				&& (PreviousClose[0] < PreviousLow[0] || PreviousClose[0] > PreviousHigh[0]))
				DrawTextFixed("errortag5", errorData5, TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
			else
				DrawTextFixed("errortag5", "", TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
		}

		#region Properties

		/// <summary>
		/// </summary>
		[Description("Select the settlement or close price used for calculating pivots.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Settlement/Close")]
		public Data.HLCCalculationMode PriorDayHLC
		{
			get { return priorDayHLC; }
			set { priorDayHLC = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Option to show MidPivots")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Midpivots")]
		public bool ShowMidpivots 
		{
			get { return showMidpivots; }
			set { showMidpivots = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Option where to plot labels")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Position")]
		public anaPlotAlignPD31 PlotLabels
		{
			get { return plotLabels; }
			set { plotLabels = value; }
		}
			
		/// <summary>
		/// </summary>
		[Description("Width of the pivot lines as # of bars.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Line Width")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(0, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Label distance from line.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Offset")]
		public int LabelPosition
		{
			get { return labelPosition; }
			set { labelPosition = Math.Max(1, value); }
		}
	
		/// <summary>
		/// </summary>
		[Description("Pivot Formula Type")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Pivot Formula")]
		public anaPivotStylesPD31 PivotFormula
		{
			get { return pivotFormula; }
			set { pivotFormula = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Session used for calculating pivots")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Session")]
		public anaSessionTypePD31 PivotSession
		{
			get { return pivotSession; }
			set { pivotSession = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Session used for calculating RTH pivots")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Session # for RTH)")]
		public anaSessionCountPD31 ActiveSession
		{
			get { return activeSession; }
			set { activeSession = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PP
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R1
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S1
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R2
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S2
		{
			get { return Values[4]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R3
		{
			get { return Values[5]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S3
		{
			get { return Values[6]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R4
		{
			get { return Values[7]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S4
		{
			get { return Values[8]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries RMid
		{
			get { return Values[9]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries SMid
		{
			get { return Values[10]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R12Mid
		{
			get { return Values[11]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S12Mid
		{
			get { return Values[12]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R23Mid
		{
			get { return Values[13]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S23Mid
		{
			get { return Values[14]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries R34Mid
		{
			get { return Values[15]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries S34Mid
		{
			get { return Values[16]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousHigh
		{
			get { return Values[17]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousLow
		{
			get { return Values[18]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PreviousClose
		{
			get { return Values[19]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries MidLine
		{
			get { return Values[20]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PivotRangeLine
		{
			get { return Values[21]; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries PVWAP
		{
			get { return Values[22]; }
		}

		/// <summary>
		/// close value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session close value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedClose
		{
			get { return userDefinedClose; }
			set { userDefinedClose = value; }
		}

		/// <summary>
		/// high value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session high value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedHigh
		{
			get { return userDefinedHigh; }
			set { userDefinedHigh = value; }
		}

		/// <summary>
		/// low value for user defined pivots calculation
		/// </summary>
		[Description("User defined prior session low value used for pivots calculation.")]
		[GridCategory("\rUser defined values")]
		public double UserDefinedLow
		{
			get { return userDefinedLow; }
			set { userDefinedLow = value; }
		}

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
        [Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 01 /No Trade Date")]
		public DateTime PublicHoliday0
		{
			get { return publicHoliday0;}
			set { publicHoliday0 = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
        [Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 02 /No Trade Date")]
		public DateTime PublicHoliday1
		{
			get { return publicHoliday1;}
			set { publicHoliday1 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 03 /No Trade Date")]
		public DateTime PublicHoliday2
		{
			get { return publicHoliday2;}
			set { publicHoliday2 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 04 /No Trade Date")]
		public DateTime PublicHoliday3
		{
			get { return publicHoliday3;}
			set { publicHoliday3 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 05 /No Trade Date")]
		public DateTime PublicHoliday4
		{
			get { return publicHoliday4;}
			set { publicHoliday4 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 06 /No Trade Date")]
		public DateTime PublicHoliday5
		{
			get { return publicHoliday5;}
			set { publicHoliday5 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 07 /No Trade Date")]
		public DateTime PublicHoliday6
		{
			get { return publicHoliday6;}
			set { publicHoliday6 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 08 /No Trade Date")]
		public DateTime PublicHoliday7
		{
			get { return publicHoliday7;}
			set { publicHoliday7 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 09 /No Trade Date")]
		public DateTime PublicHoliday8
		{
			get { return publicHoliday8;}
			set { publicHoliday8 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 10 /No Trade Date")]
		public DateTime PublicHoliday9
		{
			get { return publicHoliday9;}
			set { publicHoliday9 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 11 /No Trade Date")]
		public DateTime PublicHoliday10
		{
			get { return publicHoliday10;}
			set { publicHoliday10 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 12 /No Trade Date")]
		public DateTime PublicHoliday11
		{
			get { return publicHoliday11;}
			set { publicHoliday11 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 13 /No Trade Date")]
		public DateTime PublicHoliday12
		{
			get { return publicHoliday12;}
			set { publicHoliday12 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 14 /No Trade Date")]
		public DateTime PublicHoliday13
		{
			get { return publicHoliday13;}
			set { publicHoliday13 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 15 /No Trade Date")]
		public DateTime PublicHoliday14
		{
			get { return publicHoliday14;}
			set { publicHoliday14 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 16 /No Trade Date")]
		public DateTime PublicHoliday15
		{
			get { return publicHoliday15;}
			set { publicHoliday15 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 17 /No Trade Date")]
		public DateTime PublicHoliday16
		{
			get { return publicHoliday16;}
			set { publicHoliday16 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 18 /No Trade Date")]
		public DateTime PublicHoliday17
		{
			get { return publicHoliday17;}
			set { publicHoliday17 = value;}
		}	
		#endregion

		#region Miscellaneous

		public override string FormatPriceMarker(double price)
		{
			double trunc = Math.Truncate(price);
			int fraction = Convert.ToInt32(320 * Math.Abs(price - trunc) - 0.0001); // rounding down for ZF and ZT
			string priceMarker = "";
			if (TickSize == 0.03125) 
			{
				fraction = fraction/10;
				if (fraction < 10)
					priceMarker = trunc.ToString() + "'0" + fraction.ToString();
				else 
					priceMarker = trunc.ToString() + "'" + fraction.ToString();
			}
			else if (TickSize == 0.015625 || TickSize == 0.0078125)
			{
				if (fraction < 10)
					priceMarker = trunc.ToString() + "'00" + fraction.ToString();
				else if (fraction < 100)
					priceMarker = trunc.ToString() + "'0" + fraction.ToString();
				else	
					priceMarker = trunc.ToString() + "'" + fraction.ToString();
			}
			else
				priceMarker = price.ToString(Gui.Globals.GetTickFormatString(TickSize));
			return priceMarker;
		}		

		/// <summary>
		/// Have the .GetBars() call in a seperate thread. Reason: NT internal pool locking would not work if called in main thread.
		/// </summary>
		/// <param name="state"></param>
		private void GetBarsNow(object state)
		{
			if (Disposed)
				return;

			dailyBars			= Data.Bars.GetBars(Bars.Instrument, new Period(PeriodType.Day, 1, Bars.Period.MarketDataType), Bars.From, Bars.To, (Session) Bars.Session.Clone(), Data.Bars.SplitAdjust, Data.Bars.DividendAdjust);
			existsHistDailyData	= (dailyBars.Count <= 1) ? false : true;
			isDailyDataLoaded	= true;
			Enabled				= true;

			Cbi.Globals.SynchronizeInvoke.AsyncInvoke(new System.Windows.Forms.MethodInvoker(InvalidateNow), null);
		}

		private DateTime GetLastBarSessionDate(DateTime time, Data.Bars bars, PivotRange pivotRange)
		{
			if (time > cacheSessionEndTmp)
			{	
				isPublicHoliday = false;
				for (int i =0; i<18; i++)
				{
					if (publicHoliday[i].Date == sessionDateTmp)
						isPublicHoliday = true;
				}
				Bars.Session.GetNextBeginEnd(time, out cacheSessionBeginTmp, out cacheSessionEndTmp);
				if (Bars.BarsType.IsIntraday)
					sessionDateTmp = Bars.GetTradingDayFromLocal(time);
				else
					sessionDateTmp = time.Date;
				
				if(cacheSessionDate != sessionDateTmp) 
					{
						cacheSessionDate = sessionDateTmp;
						if (!isPublicHoliday && (newSessionBarIdxArr1.Count == 0 
							|| (newSessionBarIdxArr1.Count > 0 && CurrentBar > (int) newSessionBarIdxArr1[newSessionBarIdxArr1.Count - 1])))
								newSessionBarIdxArr1.Add(CurrentBar);
					}
			}
			return sessionDateTmp;
		}
		

		internal void InvalidateNow()
		{
			if (Disposed || ChartControl == null)
				return;

			ChartControl.Invalidate(true);
		}

		/// <summary>
        /// Overload this method to handle the termination of an indicator. Use this method to dispose of any resources vs overloading the Dispose() method.
		/// </summary>
		protected override void OnTermination()
		{
			if (dailyBars != null)
				dailyBars.Dispose();

			textBrush.Dispose();
			foreach (SolidBrush solidBrush in brushes)
				solidBrush.Dispose();
			stringFormatCenter.Dispose();
			stringFormatFar.Dispose();	
			stringFormatNear.Dispose();
		}
	
		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;
			// plot error if data not complete
			DateTime lastBarSessionBegin = Cbi.Globals.MinDate;
			DateTime lastBarSessionEnd = Cbi.Globals.MinDate;
			Bars.Session.GetNextBeginEnd(Bars.Get(Math.Min(Bars.Count - 1, this.LastBarIndexPainted)).Time, out lastBarSessionBegin, out lastBarSessionEnd);
			textBrush.Color = ChartControl.AxisColor;
			SizeF errorSize	= graphics.MeasureString(errorData4, textFont);
			errorTextWidth	= errorSize.Width + 5;
			errorTextHeight	= errorSize.Height + 5;
			if (Bars.Count > 0 && plotStart.Date >= lastBarSessionEnd.Date && priorDayHLC != HLCCalculationMode.UserDefinedValues)
				graphics.DrawString(errorData4, textFont, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);

			int	barWidth = ChartControl.ChartStyle.GetBarPaintWidth(Bars.BarsData.ChartStyle.BarWidthUI);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				int				firstBarIdxToPaint	= -1;
				int				lastX				= -1;
				int				lastY				= -1;
				int				firstX				= -1;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries		series				= (DataSeries) Values[seriesCount];

				for (int i = newSessionBarIdxArr1.Count - 1; i >= 0; i--)
				{
					int prevSessionBreakIdx = (int) newSessionBarIdxArr1[i];
					if (prevSessionBreakIdx <= this.LastBarIndexPainted)
					{
						firstBarIdxToPaint = prevSessionBreakIdx;
						break;
					}
				}

				using (GraphicsPath	path = new GraphicsPath()) 
				{
					if (brush.Color != plot.Pen.Color)	
						brush = new SolidBrush(plot.Pen.Color);

					for (int idx = this.LastBarIndexPainted; idx >= Math.Max(this.FirstBarIndexPainted, this.LastBarIndexPainted - Width); idx--)
					{
						if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
							continue;
						else if (!series.IsValidPlot(idx))
							continue;

						if (idx < firstBarIdxToPaint)
							break;

						double	val = series.Get(idx);
						int		x	= ChartControl.GetXByBarIdx(BarsArray[0], idx);
						int		y	= ChartControl.GetYByValue(this, val);

						if (lastX >= 0)
						{
							if (y != lastY) // Problem here is, that last bar of old day has date of new day
								y = lastY;
							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
						lastX	= x;
						lastY	= y;
						if (idx == this.LastBarIndexPainted || idx == this.LastBarIndexPainted-1)
							firstX	= x;
					}

					graphics.SmoothingMode = SmoothingMode.AntiAlias;
					graphics.DrawPath(plot.Pen, path);
					graphics.SmoothingMode = oldSmoothingMode;
					if (PlotLabels == anaPlotAlignPD31.Right)
					graphics.DrawString(plot.Name, ChartControl.Font, brush, firstX + LabelPosition + 30, lastY - ChartControl.Font.GetHeight() / 2, stringFormatNear);
					if (PlotLabels == anaPlotAlignPD31.Left)
					graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX - LabelPosition, lastY - ChartControl.Font.GetHeight() / 2, stringFormatFar);
				}
			}
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6 - (((int) time.DayOfWeek) + 1) % 7));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}
		#endregion
	}
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaPivotsDailyV31[] cacheanaPivotsDailyV31 = null;

        private static anaPivotsDailyV31 checkanaPivotsDailyV31 = new anaPivotsDailyV31();

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public anaPivotsDailyV31 anaPivotsDailyV31(anaSessionCountPD31 activeSession, int labelPosition, anaPivotStylesPD31 pivotFormula, anaSessionTypePD31 pivotSession, anaPlotAlignPD31 plotLabels, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return anaPivotsDailyV31(Input, activeSession, labelPosition, pivotFormula, pivotSession, plotLabels, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public anaPivotsDailyV31 anaPivotsDailyV31(Data.IDataSeries input, anaSessionCountPD31 activeSession, int labelPosition, anaPivotStylesPD31 pivotFormula, anaSessionTypePD31 pivotSession, anaPlotAlignPD31 plotLabels, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            if (cacheanaPivotsDailyV31 != null)
                for (int idx = 0; idx < cacheanaPivotsDailyV31.Length; idx++)
                    if (cacheanaPivotsDailyV31[idx].ActiveSession == activeSession && cacheanaPivotsDailyV31[idx].LabelPosition == labelPosition && cacheanaPivotsDailyV31[idx].PivotFormula == pivotFormula && cacheanaPivotsDailyV31[idx].PivotSession == pivotSession && cacheanaPivotsDailyV31[idx].PlotLabels == plotLabels && cacheanaPivotsDailyV31[idx].PriorDayHLC == priorDayHLC && cacheanaPivotsDailyV31[idx].ShowMidpivots == showMidpivots && Math.Abs(cacheanaPivotsDailyV31[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV31[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV31[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cacheanaPivotsDailyV31[idx].Width == width && cacheanaPivotsDailyV31[idx].EqualsInput(input))
                        return cacheanaPivotsDailyV31[idx];

            lock (checkanaPivotsDailyV31)
            {
                checkanaPivotsDailyV31.ActiveSession = activeSession;
                activeSession = checkanaPivotsDailyV31.ActiveSession;
                checkanaPivotsDailyV31.LabelPosition = labelPosition;
                labelPosition = checkanaPivotsDailyV31.LabelPosition;
                checkanaPivotsDailyV31.PivotFormula = pivotFormula;
                pivotFormula = checkanaPivotsDailyV31.PivotFormula;
                checkanaPivotsDailyV31.PivotSession = pivotSession;
                pivotSession = checkanaPivotsDailyV31.PivotSession;
                checkanaPivotsDailyV31.PlotLabels = plotLabels;
                plotLabels = checkanaPivotsDailyV31.PlotLabels;
                checkanaPivotsDailyV31.PriorDayHLC = priorDayHLC;
                priorDayHLC = checkanaPivotsDailyV31.PriorDayHLC;
                checkanaPivotsDailyV31.ShowMidpivots = showMidpivots;
                showMidpivots = checkanaPivotsDailyV31.ShowMidpivots;
                checkanaPivotsDailyV31.UserDefinedClose = userDefinedClose;
                userDefinedClose = checkanaPivotsDailyV31.UserDefinedClose;
                checkanaPivotsDailyV31.UserDefinedHigh = userDefinedHigh;
                userDefinedHigh = checkanaPivotsDailyV31.UserDefinedHigh;
                checkanaPivotsDailyV31.UserDefinedLow = userDefinedLow;
                userDefinedLow = checkanaPivotsDailyV31.UserDefinedLow;
                checkanaPivotsDailyV31.Width = width;
                width = checkanaPivotsDailyV31.Width;

                if (cacheanaPivotsDailyV31 != null)
                    for (int idx = 0; idx < cacheanaPivotsDailyV31.Length; idx++)
                        if (cacheanaPivotsDailyV31[idx].ActiveSession == activeSession && cacheanaPivotsDailyV31[idx].LabelPosition == labelPosition && cacheanaPivotsDailyV31[idx].PivotFormula == pivotFormula && cacheanaPivotsDailyV31[idx].PivotSession == pivotSession && cacheanaPivotsDailyV31[idx].PlotLabels == plotLabels && cacheanaPivotsDailyV31[idx].PriorDayHLC == priorDayHLC && cacheanaPivotsDailyV31[idx].ShowMidpivots == showMidpivots && Math.Abs(cacheanaPivotsDailyV31[idx].UserDefinedClose - userDefinedClose) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV31[idx].UserDefinedHigh - userDefinedHigh) <= double.Epsilon && Math.Abs(cacheanaPivotsDailyV31[idx].UserDefinedLow - userDefinedLow) <= double.Epsilon && cacheanaPivotsDailyV31[idx].Width == width && cacheanaPivotsDailyV31[idx].EqualsInput(input))
                            return cacheanaPivotsDailyV31[idx];

                anaPivotsDailyV31 indicator = new anaPivotsDailyV31();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ActiveSession = activeSession;
                indicator.LabelPosition = labelPosition;
                indicator.PivotFormula = pivotFormula;
                indicator.PivotSession = pivotSession;
                indicator.PlotLabels = plotLabels;
                indicator.PriorDayHLC = priorDayHLC;
                indicator.ShowMidpivots = showMidpivots;
                indicator.UserDefinedClose = userDefinedClose;
                indicator.UserDefinedHigh = userDefinedHigh;
                indicator.UserDefinedLow = userDefinedLow;
                indicator.Width = width;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaPivotsDailyV31[] tmp = new anaPivotsDailyV31[cacheanaPivotsDailyV31 == null ? 1 : cacheanaPivotsDailyV31.Length + 1];
                if (cacheanaPivotsDailyV31 != null)
                    cacheanaPivotsDailyV31.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaPivotsDailyV31 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaPivotsDailyV31 anaPivotsDailyV31(anaSessionCountPD31 activeSession, int labelPosition, anaPivotStylesPD31 pivotFormula, anaSessionTypePD31 pivotSession, anaPlotAlignPD31 plotLabels, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.anaPivotsDailyV31(Input, activeSession, labelPosition, pivotFormula, pivotSession, plotLabels, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaPivotsDailyV31 anaPivotsDailyV31(Data.IDataSeries input, anaSessionCountPD31 activeSession, int labelPosition, anaPivotStylesPD31 pivotFormula, anaSessionTypePD31 pivotSession, anaPlotAlignPD31 plotLabels, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.anaPivotsDailyV31(input, activeSession, labelPosition, pivotFormula, pivotSession, plotLabels, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaPivotsDailyV31 anaPivotsDailyV31(anaSessionCountPD31 activeSession, int labelPosition, anaPivotStylesPD31 pivotFormula, anaSessionTypePD31 pivotSession, anaPlotAlignPD31 plotLabels, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            return _indicator.anaPivotsDailyV31(Input, activeSession, labelPosition, pivotFormula, pivotSession, plotLabels, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }

        /// <summary>
        /// Daily Session Pivots.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaPivotsDailyV31 anaPivotsDailyV31(Data.IDataSeries input, anaSessionCountPD31 activeSession, int labelPosition, anaPivotStylesPD31 pivotFormula, anaSessionTypePD31 pivotSession, anaPlotAlignPD31 plotLabels, Data.HLCCalculationMode priorDayHLC, bool showMidpivots, double userDefinedClose, double userDefinedHigh, double userDefinedLow, int width)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaPivotsDailyV31(input, activeSession, labelPosition, pivotFormula, pivotSession, plotLabels, priorDayHLC, showMidpivots, userDefinedClose, userDefinedHigh, userDefinedLow, width);
        }
    }
}
#endregion
